import 'package:flutter/widgets.dart';
import 'package:observable_pattern/src/observable.dart';

///a class that receive an observable and a builder to rebuild when a value change from Observable object
class Observer<T, E> extends StatefulWidget {
  final Observable<T, E> observable;
  final AsyncWidgetBuilder<T> builder;

  const Observer({
    Key key,
    this.observable,
    @required this.builder,
  })  : assert(builder != null),
        super(key: key);

  @override
  _ObserverState<T, E> createState() => _ObserverState<T, E>();
}

class _ObserverState<T, E> extends State<Observer<T, E>> {
  AsyncSnapshot<T> _snapshot;

  @override
  void initState() {
    super.initState();
    _snapshot = AsyncSnapshot<T>.withData(
        ConnectionState.none, widget.observable?.value);
    WidgetsBinding.instance.addPostFrameCallback(callback);
  }

  void callback(_) {
    widget.observable?.addListener(init);
  }

  void init(_) {
    if (mounted) {
      if (widget.observable.hasError) {
        _snapshot = AsyncSnapshot<T>.withError(
            ConnectionState.done, widget.observable.error);
      } else {
        _snapshot = AsyncSnapshot.withData(
            ConnectionState.done, widget.observable.value);
      }
      setState(() {});
    }
  }

  @override
  void didUpdateWidget(Observer<T, E> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.observable != widget.observable) {
      oldWidget.observable?.removeListener(init);
      widget.observable?.addListener(init);
    }
  }

  @override
  void dispose() {
    widget.observable?.removeListener(init);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final child = widget.builder(context, _snapshot);
    assert(child != null);
    return child;
  }
}
